﻿using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Atelier.Reports;

internal class DocReport
{
    private readonly IClientRepository _clientRepository;
    private readonly IFabricRepository _fabricRepository;
    private readonly IModelRepository _modelRepository;
    private readonly ILogger<DocReport> _logger;
    public DocReport(IClientRepository clientRepository, IFabricRepository fabricRepository, IModelRepository modelRepository, ILogger<DocReport> logger)
    {
        _clientRepository = clientRepository ?? throw new ArgumentNullException(nameof(clientRepository));
        _fabricRepository = fabricRepository ?? throw new ArgumentNullException(nameof(fabricRepository));
        _modelRepository = modelRepository ?? throw new ArgumentNullException(nameof(modelRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }
    public bool CreateDoc(string filePath, bool includeClients, bool includeFabrics, bool includeModels)
    {
        try
        {
            var builder = new WordBuilder(filePath)
            .AddHeader("Документ со справочниками");
            if (includeClients)
            {
                builder.AddParagraph("Клиенты")
                .AddTable([2400, 2400, 2400], GetClients());
            }
            if (includeFabrics)
            {
                builder.AddParagraph("Ткани")
                .AddTable([2400, 2400, 2400], GetFabrics());
            }
            if (includeModels)
            {
                builder.AddParagraph("Модели")
                .AddTable([2400, 2400, 1200, 1200], GetModels());
            }
            builder.Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }
    private List<string[]> GetClients()
    {
        return [
        ["Имя", "Фамилия", "Контактные данные"],
.. _clientRepository
.ReadClients()
.Select(x => new string[] { x.FirstName, x.LastName, x.ContactInformation }),
];
    }
    private List<string[]> GetFabrics()
    {
        return [
        ["Тип ткани", "Цвет", "Метраж"],
.. _fabricRepository
.ReadFabrics()
.Select(x => new string[] { x.FabricType.ToString(), x.Color.ToString(), x.Metrage.ToString() }),
];
    }
    private List<string[]> GetModels()
    {
        // Заголовок таблицы
        var result = new List<string[]>
    {
        new[] { "Тип модели", "Цена", "Ткань", "Количество" }
    };

        // Добавляем строки с данными
        var models = _modelRepository.ReadModels().ToList();
        _logger.LogDebug("Models: {Models}", JsonConvert.SerializeObject(models));

        result.AddRange(
            models.Select(x =>
            {
                // Формируем строки для каждой модели
                var fabricTypes = string.Join(", ", x.FabricModel
                    .Select(y => _fabricRepository.ReadFabricById(y.FabricId)?.FabricType)
                    .Where(type => type != null)); // Filter out null types
                var fabricCounts = string.Join(", ", x.FabricModel.Select(y => y.Count.ToString()));
                _logger.LogDebug("ModelType: {ModelType}, Price: {Price}, FabricTypes: {FabricTypes}, FabricCounts: {FabricCounts}", x.ModelType, x.Price, fabricTypes, fabricCounts);

                return new[]
                {
                x.ModelType.ToString(),
                x.Price.ToString(),
                fabricTypes,
                fabricCounts
                };
            })
        );

        return result;
    }
}