﻿using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Atelier.Reports
{
    internal class TableReport
    {
        private readonly IOrderRepository _orderRepository;
        private readonly IStorageRepository _storageRepository;
        private readonly ILogger<TableReport> _logger;
        internal static readonly string[] item = ["Клиент", "Дата", "Модель", "Количество"];

        public TableReport(IOrderRepository orderRepository, IStorageRepository storageRepository, ILogger<TableReport> logger)
        {
            _orderRepository = orderRepository ?? throw new ArgumentNullException(nameof(orderRepository));
            _storageRepository = storageRepository ?? throw new ArgumentNullException(nameof(storageRepository));
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
        }

        public bool CreateTable(string filePath, int modelId, DateTime startDate, DateTime endDate)
        {
            try
            {
                new ExcelBuilder(filePath)
                    .AddHeader("Сводка по заказам", 0, 4)
                    .AddParagraph("за период", 0)
                    .AddTable(new[] { 15, 15, 20, 15 }, GetData(modelId, startDate, endDate))
                    .Build();
                return true;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при формировании документа");
                return false;
            }
        }

        private List<string[]> GetData(int modelId, DateTime startDate, DateTime endDate)
        {
            var orderData = _orderRepository
                .ReadOrders()
                .Where(x => x.Date >= startDate && x.Date <= endDate && x.ModelOrder.Any(y => y.ModelId == modelId))
                .SelectMany(x => x.ModelOrder, (order, modelOrder) => new { order.ClientId, order.Date, modelOrder.ModelId, modelOrder.Count })
                .OrderBy(x => x.Date);

            var storageData = _storageRepository
                .ReadStorage()
                .Where(x => x.DateStorage >= startDate && x.DateStorage <= endDate)
                .Select(x => new { x.FabricId, x.DateStorage, x.StockMetrage })
                .OrderBy(x => x.DateStorage);

            _logger.LogDebug("Order Data: {OrderData}", JsonConvert.SerializeObject(orderData));
            _logger.LogDebug("Storage Data: {StorageData}", JsonConvert.SerializeObject(storageData));

            var result = new List<string[]>() { item };

            result.AddRange(orderData.Select(x => new string[] { x.ClientId.ToString(), x.Date.ToString("yyyy-MM-dd"), x.ModelId.ToString(), x.Count.ToString() }));
            result.AddRange(storageData.Select(x => new string[] { "", x.DateStorage.ToString("yyyy-MM-dd"), x.FabricId.ToString(), x.StockMetrage.ToString() }));

            result.Add(new[] { "Всего", "", "", (orderData.Sum(x => x.Count) + storageData.Sum(x => x.StockMetrage)).ToString() });

            return result;
        }
    }
}
