﻿using Atelier.Repositories;
using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using System.Collections.Generic;

internal class ModelRepository : IModelRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<ModelRepository> _logger;

    public ModelRepository(IConnectionString connectionString, ILogger<ModelRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateModel(Model model)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(model));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();

            var queryInsert = @"
            INSERT INTO Models (ModelType, Price)
            VALUES (@ModelType, @Price)
            RETURNING Id";

            var modelId = connection.QueryFirst<int>(queryInsert, model, transaction);

            var querySubInsert = @"
            INSERT INTO FabricModel (ModelId, FabricId, Count)
            VALUES (@ModelId, @FabricId, @Count)";

            foreach (var elem in model.FabricModel)
            {
                connection.Execute(querySubInsert, new
                {
                    ModelId = modelId,
                    elem.FabricId,
                    elem.Count
                }, transaction);
            }

            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public void UpdateModel(Model model)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(model));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();

            var queryUpdateModel = @"
            UPDATE Models
            SET ModelType = @ModelType, Price = @Price
            WHERE Id = @Id";

            connection.Execute(queryUpdateModel, model, transaction);

            var queryDeleteFabricModel = @"
            DELETE FROM FabricModel
            WHERE ModelId = @Id";

            connection.Execute(queryDeleteFabricModel, new { model.Id }, transaction);

            var queryInsertFabricModel = @"
            INSERT INTO FabricModel (ModelId, FabricId, Count)
            VALUES (@ModelId, @FabricId, @Count)";

            foreach (var elem in model.FabricModel)
            {
                connection.Execute(queryInsertFabricModel, new
                {
                    ModelId = model.Id,
                    elem.FabricId,
                    elem.Count
                }, transaction);
            }

            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }

    public void DeleteModel(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
            DELETE FROM Models
            WHERE Id = @id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public Model ReadModelById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"SELECT * FROM Models
                            WHERE Id = @id";
            var model = connection.QueryFirstOrDefault<Model>(querySelect, new { id });
            var querySelectSub = @$"SELECT *
                                   FROM FabricModel fm
                                   WHERE ModelId = {id}";
            var fabricModel = connection.Query<FabricModel>(querySelectSub);
            model.CreateList(fabricModel);
            if (model == null)
            {
                _logger.LogWarning("Объект с идентификатором {id} не найден", id);
                throw new KeyNotFoundException($"Модель с идентификатором {id} не найдена");
            }
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(model));


            return model;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }



    public IEnumerable<Model> ReadModels()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM Models";
            var models = connection.Query<Model>(querySelect).ToList();

            foreach (var model in models)
            {
                var querySelectSub = @$"SELECT * FROM FabricModel fm WHERE ModelId = {model.Id}";
                var fabricModel = connection.Query<FabricModel>(querySelectSub).ToList();
                model.CreateList(fabricModel);
                _logger.LogDebug("Model ID: {ModelId}, FabricModels: {FabricModels}", model.Id, JsonConvert.SerializeObject(fabricModel));
            }

            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(models));
            return models;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
